/*
	$Id: StartStopServlet.java 6629 2014-11-27 07:41:11Z nicklas $

	Copyright (C) Nicklas Nordborg
	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.clients.web.servlet;

import javax.servlet.ServletContextEvent;
import javax.servlet.ServletContextListener;

import net.sf.basedb.clients.web.extensions.ExtensionsControl;
import net.sf.basedb.core.Application;

/**
	Servlet listener for starting BASE when the web server is started and
	stopping BASE when the web server is stopped. This servlet is very simple.
	It will call {@link Application#start()} and {@link ExtensionsControl#init(javax.servlet.ServletContext)}
	when it is loaded and {@link Application#stop()} and {@link ExtensionsControl#close()}
	when it is destroyed.

	
	@author nicklas
	@version 2.8
	@base.modified $Date: 2014-11-27 08:41:11 +0100 (to, 27 nov 2014) $
*/
public class StartStopServlet 
	implements ServletContextListener
{
	
	private static final long serialVersionUID = 8267932757094628788L;

	public StartStopServlet()
	{}
	
	@Override
	public void contextInitialized(ServletContextEvent ctx) 
	{
		Application.start();
		ExtensionsControl.init(ctx.getServletContext());
	}
	
	@Override
	public void contextDestroyed(ServletContextEvent ctx) 
	{
		ExtensionsControl.close();
		Application.stop();		
	}


}
