/**
	$Id: ParameterInfo.java 5437 2010-10-01 11:51:57Z nicklas $

	Copyright (C) 2006 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core;

import java.util.Collections;
import java.util.List;

import net.sf.basedb.core.data.ParameterValueData;

/**
	Get information about a plugin configuration or parameter. The information
	include the label, description and values of the parameter.

	@author nicklas
	@version 2.0
	@base.modified $Date: 2010-10-01 13:51:57 +0200 (fr, 01 okt 2010) $
*/
public class ParameterInfo
{
	private final String label;
	private final String description;
	private final List<?> values;
	private final Type valueType;

	ParameterInfo(DbControl dc, ParameterValueData parameter)
	{
		this.label = parameter.getLabel();
		this.description = parameter.getDescription();
		this.valueType = parameter.getValueType() == 0 ? null : Type.fromValue(parameter.getValueType());
		this.values = Collections.unmodifiableList(Values.getItemValues(dc, parameter.getValues()));
	}
	
	public String getLabel()
	{
		return label;
	}
	
	public String getDescription()
	{
		return description;
	}
	
	/**
		Get the matching {@link Type} for the values stored in
		this parameter. Can be null since not all parameter types have
		a matching value type.
		@return A Type object or null
		@since 2.16
	*/
	public Type getValueType()
	{
		return valueType;
	}
	
	public List<?> getValues()
	{
		return values;
	}
	
}
