/*
	$Id: PluginTypeData.java 6127 2012-09-14 07:28:35Z nicklas $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.data;

import java.util.Date;
import java.util.Set;

/**
	This class represent an interface that a plugin can implement.

	@author Nicklas
	@version 2.0
	@see net.sf.basedb.core.PluginType
	@see <a href="../../../../../../html/developer/api/data_api.plugins.html">Developer documentation: Plug-ins, jobs and job agents</a>
	@hibernate.class table="`PluginTypes`" lazy="false"
	@base.modified $Date: 2012-09-14 09:28:35 +0200 (fr, 14 sep 2012) $
*/
public class PluginTypeData
	extends BasicData
	implements NameableData, RemovableData, RegisteredData
{

	public PluginTypeData()
	{}

	/*
		From the NameableData interface
		-------------------------------------------
	*/
	private String name;
	@Override
	public String getName()
	{
		return name;
	}
	@Override
	public void setName(String name)
	{
		this.name = name;
	}
	private String description;
	@Override
	public String getDescription()
	{
		return description;
	}
	@Override
	public void setDescription(String description)
	{
		this.description = description;
	}
	// -------------------------------------------
	/*
		From the RemovableData interface
		-------------------------------------------
	*/
	private boolean removed;
	@Override
	public boolean isRemoved()
	{
		return removed;
	}
	@Override
	public void setRemoved(boolean removed)
	{
		this.removed = removed;
	}
	// -------------------------------------------
	/*
		From the RegisteredData interface
		-------------------------------------------
	*/
	private Date entryDate;
	@Override
	public Date getEntryDate()
	{
		return entryDate;
	}
	public void setEntryDate(Date entryDate)
	{
		this.entryDate = entryDate;
	}
	// -------------------------------------------
	/**
		The maximum length of the interface name that can be stored in the
		database.
	*/
	public static final int MAX_INTERFACENAME_LENGTH = 255;
	private String interfaceName;
	/**
		Get the interface name a plugin must implement to be of this type.
		The specified interface must extend the {@link net.sf.basedb.core.plugin.Plugin}
		interface.
		@hibernate.property column="`interface_name`" type="string" length="255" not-null="true" unique="true"
	*/
	public String getInterfaceName()
	{
		return interfaceName;
	}
	public void setInterfaceName(String interfaceName)
	{
		this.interfaceName = interfaceName;
	}

	public static final int MAX_JARFILE_LENGTH = 255;
	private String jarFile;
	/**
		Get the file name of the JAR file where the plugin class is located.
		The JAR file must be located in the 'plugins.dir' folder specified
		in base.config. If the value is null the plugin must be in the 
		classpath (eg. WEB-INF/lib).
		@return The class name for this plugin
		@hibernate.property column="`jar_file`" type="string" length="255" not-null="false"
		@since 3.0
	*/
	public String getJarFile()
	{
		return jarFile;
	}
	public void setJarFile(String jarFile)
	{
		this.jarFile = jarFile;
	}
	
	private Set<PluginDefinitionData> plugins;
	/**
		The plugin implementing this interface. This is the inverse end.
		@see PluginDefinitionData#getPluginTypes()
		@hibernate.set table="`PluginDefinitionTypes`" lazy="true"
		@hibernate.collection-key column="`plugintype_id`"
		@hibernate.collection-many-to-many column="`plugindefinition_id`" class="net.sf.basedb.core.data.PluginDefinitionData"
	*/
	Set<PluginDefinitionData> getPlugins()
	{
		return plugins;
	}
	void setPlugins(Set<PluginDefinitionData> plugins)
	{
		this.plugins = plugins;
	}
}
