/**
	$Id: ListUtil.java 6871 2015-04-16 11:53:37Z nicklas $

	Copyright (C) 2008 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.biomaterial;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;

import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.BioMaterial;
import net.sf.basedb.core.BioSource;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Extract;
import net.sf.basedb.core.InvalidDataException;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.ItemList;
import net.sf.basedb.core.ItemQuery;
import net.sf.basedb.core.PermissionDeniedException;
import net.sf.basedb.core.Sample;
import net.sf.basedb.util.collections.AddAllTransformer;
import net.sf.basedb.util.collections.CollectionTransformer;
import net.sf.basedb.util.collections.MultiStepCollectionTransformer;
import net.sf.basedb.util.listable.ListableUtil;
import net.sf.basedb.util.listable.SourceItemTransformerFactory;

/**
	Utility class for working with biomaterials.
	
	@author Nicklas
	@version 2.10
	@base.modified $Date: 2015-04-16 13:53:37 +0200 (to, 16 apr 2015) $
	@deprecated In 3.5, for alternatives see documentation for each method
*/
@Deprecated
public class ListUtil
{

	/**
		Get a query that returns all biomaterials that can possibly be 
		members of this list. NOTE! This is just a convenience function
		that calls {@link BioSource#getQuery()}, etc. depending on the
		member type of the list.
		@return An {@link ItemQuery} object
		@throws BaseException If the query could not be created and configured.
		@deprecated In 3.5, use {@link Item#getQuery()} instead
	*/
	@Deprecated
	public static ItemQuery<? extends BioMaterial> getQuery(Item biomaterialType)
		throws BaseException
	{
		ItemQuery<? extends BioMaterial> query;
		if (biomaterialType == Item.BIOSOURCE)
		{
			query = BioSource.getQuery();
		}
		else if (biomaterialType == Item.SAMPLE)
		{
			query = Sample.getQuery();
		}
		else if (biomaterialType == Item.EXTRACT)
		{
			query = Extract.getQuery();
		}
		else
		{
			throw new AssertionError("Can't create query for item type: " + 
					biomaterialType);
		}
		return query;
	}
	
	
	/**
		Adds items to a biomaterial list. The source collection can optionally
		be transformed by a transformer. If no transformer is given, the
		source collection is added. The items added to the list must of course
		be of the correct {@link ItemList#getMemberType()} for the list.
		
		@param list The list to add items to
		@param sources The source collection of biomaterials
		@param transformer An optional transformer
		@return The number of items added to the biomaterial list
		@throws InvalidDataException If an item is not of the correct
			member type
		@throws PermissionDeniedException If the logged in user doesn't have
			write permission for the list
		@deprecated In 3.5, no replacement
	*/
	@Deprecated
	public static <S extends BioMaterial> int addToList(ItemList list, 
		Collection<? extends S> sources, 
		CollectionTransformer<S, ? extends BioMaterial> transformer)
		throws InvalidDataException, PermissionDeniedException
	{
		Iterator<? extends BioMaterial> iterator = null;
		if (transformer != null)
		{
			Collection<BioMaterial> transformed = new HashSet<BioMaterial>();
			transformer.transform(sources, transformed);
			iterator = transformed.iterator();
		}
		else
		{
			iterator = sources.iterator();
		}
		int added = list.add(iterator);
		return added;
	}
	
	/**
		Create a transformer that can transform a collection with
		<code>sourceItem</code>:s into a collection with <code>destinationItem</code>:s
		by following parent/child paths. Both the source and destination must be
		one of {@link Item#BIOSOURCE}, {@link Item#SAMPLE} or {@link Item#EXTRACT}.
		
		@param dc The DbControl the transformers should use for loading
			items from the database
		@param sourceItem The type of source items
		@param destinationItem The type of destination items
		@param sameType If children/parents of the same type should be considered or not
		@return A collection transformer (never null)
		@deprecated In 3.5, use {@link ListableUtil#getTransformerFactory(Item)} and 
			{@link SourceItemTransformerFactory#create(Item, net.sf.basedb.core.SyncFilter.SourceItemTransform)}
			instead
	*/
	@SuppressWarnings("unchecked")
	@Deprecated
	public static CollectionTransformer createTransformer(DbControl dc, Item sourceItem, Item destinationItem, boolean sameType)
	{
		// The allowed item types
		final List<Item> INDEX = Arrays.asList(new Item[] { Item.BIOSOURCE, Item.SAMPLE, Item.EXTRACT });
		int dIndex = INDEX.indexOf(destinationItem);
		if (dIndex == -1) 
		{
			throw new InvalidDataException("Destination item type must be a biomaterial: " + 
					destinationItem);
		}
		int sIndex = INDEX.indexOf(sourceItem);
		if (dIndex == -1) 
		{
			throw new InvalidDataException("Source item type must be a biomaterial: " + 
					destinationItem);
		}
		int step = sIndex < dIndex ? 1 : -1;
		
		List<CollectionTransformer> transformers = new ArrayList<CollectionTransformer>();
		while (sIndex != dIndex)
		{
			if (sourceItem == Item.BIOSOURCE)
			{
				// Moving down
				transformers.add(new BioSourceToSampleTransformer(dc));
			}
			else if (sourceItem == Item.SAMPLE)
			{
				if (step == 1)
				{
					// Moving down
					if (sameType) transformers.add(new ChildrenTransformer<Sample>(dc, true, Sample.getQuery()));
					transformers.add(new SampleToExtractTransformer(dc));
				}
				else
				{
					// Moving up
					if (sameType) transformers.add(new ParentsTransformer<Sample>(dc, true, Sample.getQuery()));
					transformers.add(new SampleToBioSourceTransformer(dc));
				}
			}
			else if (sourceItem == Item.EXTRACT)
			{
				if (step == 1)
				{
					// Moving down
					if (sameType) transformers.add(new ChildrenTransformer<Extract>(dc, true, Extract.getQuery()));
				}
				else
				{
					// Moving up
					if (sameType) transformers.add(new ParentsTransformer<Extract>(dc, true, Extract.getQuery()));
					transformers.add(new ExtractToSampleTransformer(dc));
				}
			}
			sIndex += step;
			sourceItem = INDEX.get(sIndex);
		}
		
		if (transformers.size() == 0)
		{
			// When sourceItem == destinationItem to begin with
			transformers.add(new AddAllTransformer());
		}
		
		if (sameType && destinationItem != Item.BIOSOURCE)
		{
			if (step == 1)
			{
				// Moving down
				transformers.add(new ChildrenTransformer(dc, true, getQuery(destinationItem)));
			}
			else
			{
				// Moving up
				transformers.add(new ParentsTransformer(dc, true, getQuery(destinationItem)));
			}
		}
		
		CollectionTransformer ct = null;
		if (transformers.size() == 1)
		{
			ct = transformers.get(0);
		}
		else
		{
			ct = new MultiStepCollectionTransformer(transformers);
		}
		return ct;
	}
	
}
