import java.util.Date;
import java.util.Set;

import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.BioPlateType;
import net.sf.basedb.core.BioWell;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.ItemProxy;
import net.sf.basedb.core.ItemResultList;
import net.sf.basedb.core.Permission;

/*
	$Id $

	Copyright (C) 2010 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/

public class TestBioPlateType
{

	static boolean ok = true;
	public static void main(String[] args)
	{
		TestUtil.checkArgs(args);
		TestUtil.begin();
		ok = test_all();
		TestUtil.stop();
	}

	static boolean test_all()
	{
		write("++Testing bioplate types");
		write_header();
		// Standard tests: create, load, list
		int id = test_create("Free for all", null, BioWell.LockMode.UNLOCKED);
		int id2 = test_create("Locked samples", Item.SAMPLE, BioWell.LockMode.LOCKED_AFTER_ADD);
		int id3 = test_create("Moveable samples", Item.SAMPLE, BioWell.LockMode.LOCKED_AFTER_MOVE);
		test_load(id);
		test_list(-1);

		// Standard test: Delete
		if (TestUtil.waitBeforeDelete()) TestUtil.waitForEnter();
		test_delete(id);
		test_delete(id2);
		test_delete(id3);
		write("++Testing bioplate types "+(ok ? "OK" : "Failed")+"\n");
		return ok;
	}

	static int test_create(String name, Item bioMaterialType, BioWell.LockMode lockMode)
	{
		if (!TestUtil.hasPermission(Permission.CREATE, Item.BIOPLATETYPE)) return 0;
		int id = 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			BioPlateType pt = BioPlateType.getNew(dc);
			pt.setName(name);
			pt.setDescription("Added at "+new Date());
			pt.setBioMaterialType(bioMaterialType);
			pt.setLockMode(lockMode);
			dc.saveItem(pt);
			dc.commit();
			id = pt.getId();
			write_item(0, pt);
			write("--Create bioplate type OK");
		}
		catch (Throwable ex)
		{
			write("--Create bioplate type FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}

	static void test_load(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			BioPlateType pt = BioPlateType.getById(dc, id);
			write_item(0, pt);
			write("--Load bioplate type OK");
		}
		catch (Throwable ex)
		{
			write("--Load bioplate type FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}


	static void test_list(int expectedResults)
	{
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ItemResultList<BioPlateType> l = BioPlateType.getQuery().list(dc);
			for (int i = 0; i<l.size(); i++)
			{
				write_item(i, l.get(i));
			}
			if (expectedResults >= 0 && expectedResults != l.size())
			{
				throw new BaseException("Expected "+expectedResults+" results, not "+l.size());
			}
			write("--List bioplate types OK ("+l.size()+")");
		}
		catch (Throwable ex)
		{
			write("--List bioplate types FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}


	static void test_delete(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			BioPlateType pt = BioPlateType.getById(dc, id);
			dc.deleteItem(pt);
			Set<ItemProxy> using = pt.getUsingItems();
			if (using.size() > 0) 
			{
			   throw new BaseException(pt + " is used by " + using);
			}
			dc.commit();
			write("--Delete bioplate type OK");
		}
		catch (Throwable ex)
		{
			write("--Delete bioplate type FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void write_header()
	{
		if (!TestUtil.getSilent())
		{
			System.out.println("   \tID \tName      \tDescription\tBiomaterial type\tLock mode");
			System.out.println("-- \t-- \t--------- \t-----------\t----------------\t------------");
		}
	}
	static void write_item(int i, BioPlateType pt)
		throws BaseException
	{
		if (!TestUtil.getSilent()) System.out.println(i+":\t"+pt.getId()+"\t"+pt.getName()+"\t"+pt.getDescription()+
			"\t"+pt.getBioMaterialType()+"\t"+pt.getLockMode());
	}
	static void write(String message)
	{
		System.out.println(message);
	}

}
