/**
	$Id $

	Copyright (C) 2011 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.test;

import java.util.HashMap;
import java.util.Map;

/**
	Helper class for getting parameters to import plugins for the sequencing data
	set in /testdata/sequencing.

	@author nicklas
	@since 3.0
	@base.modified $Date $
*/
public class SequenceData
{

	/**
		Get configuration parameters required for importing reporters from a GTF file.
	*/
	public static Map<String, Object> getGtfReporterParameters()
	{
		Map<String, Object> parameters = new HashMap<String, Object>();
		parameters.put("dataSplitterRegexp", "\\t");
		parameters.put("dataHeaderRegexp", "<seqname>\\t.*<transcript_id>.*");
		parameters.put("minDataColumns", 4);
		parameters.put("complexExpressions", "allow");
		parameters.put("trimQuotes", true);

		parameters.put("reporterIdColumnMapping", "\\<transcript_id>\\@\\<seqname>\\");
		parameters.put("nameColumnMapping", "\\<transcript_id>\\");
		parameters.put("symbolColumnMapping", "\\<gene_id>\\");
		parameters.put("extendedColumnMapping.chromosome", "\\<seqname>\\");
		
		return parameters;
	}

	/**
		Get configuration parameters required for importing features from a GTF file.
	*/
	public static Map<String, Object> getGtfFeatureParameters()
	{
		Map<String, Object> parameters = new HashMap<String, Object>();
		parameters.put("dataSplitterRegexp", "\\t");
		parameters.put("dataHeaderRegexp", "<seqname>\\t.*<transcript_id>.*");
		parameters.put("minDataColumns", 4);
		parameters.put("complexExpressions", "allow");
		parameters.put("trimQuotes", true);
	
		parameters.put("featureIdentification", "FEATURE_ID");
		parameters.put("reporterIdColumnMapping", "\\<transcript_id>\\@\\<seqname>\\");
		parameters.put("featureIdColumnMapping", "\\<transcript_id>\\@\\<seqname>\\");
		
		return parameters;
	}



}
