/* $Id $
	------------------------------------------------------------------
	Copyright (C) 2013 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
*/

var Samples = function()
{
	var samples = {};
	
	/**
		Initialize the page.
	*/
	samples.initPage = function()
	{
		var pageId = Doc.getPageId();
		if (pageId == 'edit-page')
		{
			// Save + Close buttons
			Buttons.addClickHandler('btnSave', samples.save);
			Buttons.addClickHandler('close', App.closeWindow);

			// Tab validation
			TabControl.addTabActivateListener('settings.annotations', samples.loadAnnotationsFrame);
			TabControl.addTabActivateListener('settings.inheritedAnnotations', samples.loadInheritedAnnotationsFrame);
			TabControl.addTabValidator('settings.info', samples.validateSample);

			// Quantity
			Events.addEventHandler('original_quantity', 'keypress', Events.numberOnly);
			
			// Protocol
			Buttons.addClickHandler('protocol_id.select', samples.selectProtocol);
			Events.addEventHandler('protocol_id', 'base-selected', samples.setProtocolCallback);
			// Subtype
			Events.addEventHandler('subtype_id', 'change', samples.subtypeOnChange);

			// Bioplate and biowell
			Buttons.addClickHandler('bioplate_id.select', samples.selectBioPlate);
			Events.addEventHandler('bioplate_id', 'base-selected', samples.setBioPlateCallback);
			Events.addEventHandler('bioplate_id', 'change', samples.bioPlateOnChange);
			Buttons.addClickHandler('biowell_id.select', samples.selectBioWell);
			Events.addEventHandler('biowell_id', 'base-selected', samples.setBioWellCallback);
			samples.initBioWell();
			
			// Parent items
			Events.addEventHandler('parentType.biosource', 'click', samples.parentTypeOnClick);
			Events.addEventHandler('parentType.sample', 'click', samples.parentTypeOnClick);
			Buttons.addClickHandler('biosource_id.select', samples.selectBioSource);
			Events.addEventHandler('biosource_id', 'base-selected', samples.setBioSourceCallback);

			Events.addEventHandler('samples', 'change', samples.samplesOnChange);
			Buttons.addClickHandler('btnAddSamples', samples.addSamplesOnClick);
			Events.addEventHandler('btnAddSamples', 'base-selected', samples.addSampleCallback);
			
			Events.addEventHandler('used_quantity', 'keypress', Events.numberOnly);
			Events.addEventHandler('used_quantity', 'keyup', samples.usedQuantityOnChange);

			samples.parentTypeOnClick();
			
		}
		else if (pageId == 'view-page')
		{
			var itemId = Data.get('page-data', 'item-id');
			var attributes = {'item-type': 'SAMPLE', 'item-id': itemId};
			Buttons.addClickHandler('btnEdit', Buttons.editItem, attributes);
			Buttons.addClickHandler('btnDelete', Buttons.deleteItem, attributes);
			Buttons.addClickHandler('btnRestore', Buttons.restoreItem, attributes);
			Buttons.addClickHandler('btnUsingItems', Buttons.showUsingItems, attributes);
			Buttons.addClickHandler('btnDeletePermanently', Buttons.deleteItemPermanently, attributes);
			Buttons.addClickHandler('btnShare', Buttons.shareItem, attributes);
			Buttons.addClickHandler('btnSetOwner', Buttons.setOwner, attributes);
			Buttons.addClickHandler('btnNewSample', samples.newSample, attributes);
			Buttons.addClickHandler('btnNewExtract', samples.newExtract, attributes);
			Buttons.addClickHandler('btnExport', Buttons.runPlugin, attributes);
			Buttons.addClickHandler('btnImport', Buttons.runPlugin, attributes);
			Buttons.addClickHandler('btnRunPlugin', Buttons.runPlugin, attributes);
			
			TabControl.addTabActivateListener('main.events', samples.viewEvents);
			TabControl.addTabActivateListener('main.annotations', AnnotationsList.loadOnce);
			TabControl.addTabActivateListener('main.overview', Overview.loadOnce);
			TabControl.addTabActivateListener('main.history', History.loadOnce);
		}
		else if (pageId == 'list-page')
		{
			var attributes = {'item-type': 'SAMPLE'};
			var tableAttributes = {'table-id': 'samples'};
			Buttons.addClickHandler('btnNewItem', Buttons.newItem, attributes);
			Buttons.addClickHandler('btnDeleteItems', Buttons.deleteItems, tableAttributes);
			Buttons.addClickHandler('btnRestoreItems', Buttons.restoreItems, tableAttributes);
			Buttons.addClickHandler('btnShareItems', Buttons.shareItems, tableAttributes);
			Buttons.addClickHandler('btnSetOwner', Buttons.setOwnerOfItems, tableAttributes);
			Buttons.addClickHandler('btnColumns', Buttons.configureColumns, tableAttributes);
			Buttons.addClickHandler('btnInheritAnnotations', Buttons.inheritAnnotations, tableAttributes);
			Buttons.addClickHandler('btnExport', Buttons.runListPlugin, tableAttributes);
			Buttons.addClickHandler('btnImport', Buttons.runListPlugin, tableAttributes);
			Buttons.addClickHandler('btnRunPlugin', Buttons.runListPlugin, tableAttributes);
			Buttons.addClickHandler('btnNewItemList', samples.newItemListOnClick);
			Buttons.addClickHandler('btnPlaceOnPlate', samples.placeOnPlateOnClick);
			Buttons.addClickHandler('btnNewPooledItem', samples.newPooledItemOnClick);
			
			Buttons.addClickHandler('close', App.closeWindow);
			Buttons.addClickHandler('btnOk', Buttons.returnSelected, tableAttributes);
		}
	}

	// Add event handlers to the 'new sample' icons
	samples.initElements = function(element, autoInit)
	{
		if (autoInit == 'new-sample')
		{
			Buttons.addClickHandler(element, samples.newSample);
		}
		else if (autoInit == 'new-extract')
		{
			Buttons.addClickHandler(element, samples.newExtract);
		}
	}

	/*
		Create a new sample with the current sample as the parent
	*/
	samples.newSample = function(event)
	{
		var sampleId = Data.int(event.currentTarget, 'item-id');
		Items.newItem('SAMPLE', '&sample_id='+sampleId);
	}
	
	/*
		Create a new extract with the current sample as the parent
	*/
	samples.newExtract = function(event)
	{
		var sampleId = Data.int(event.currentTarget, 'item-id');
		Items.newItem('EXTRACT', '&sample_id='+sampleId);
	}

	// Create a new item list with the selected items as members
	samples.newItemListOnClick = function()
	{
		var controller = Items.getController('ITEMLIST');
		Table.submitToPopup('samples', 'CreateItemList', controller.width, controller.height);
	}
	
	samples.placeOnPlateOnClick = function()
	{
		Table.submitToPopup('samples', 'PlaceOnPlate', 900, 600);
	}
	
	samples.newPooledItemOnClick = function()
	{
		if (!Table.checkIfSelected('samples'))
		{
			return;
		}
		Table.submitToPopup('samples', 'NewPooledItem', 750, 500);
	}
	
	samples.viewEvents = function()
	{
		var sampleId = Data.get('page-data', 'item-id');
		var url = '../events/index.jsp?ID='+App.getSessionId();
		url += '&cmd=List&biomaterial_id='+sampleId;
		url += '&biomaterial_type=SAMPLE';
		location.replace(url);
	}

	samples.validateSample = function()
	{
		var frm = document.forms['sample'];
		if (Strings.trim(frm.name.value) == '')
		{
			Forms.showNotification(frm.name, 'You must enter a name');
			return false;
		}
		if (Strings.trim(frm.bioplate_id.value) != 0 && Strings.trim(frm.biowell_id.value) == 0)
		{
			Forms.showNotification('biowell_id.select', 'You must choose a biowell from the bioplate');
			return false;
		}
		return true;
	}
	
	samples.save = function()
	{
		var frm = document.forms['sample'];
		if (TabControl.validateActiveTab('settings'))
		{
			Annotations.saveModifiedAnnotationsToForm(frm);
			Annotations.saveInheritedAnnotationsToForm(frm);
			Link.exportActions('samples');
			frm.submit();
		}
	}
	
	samples.selectBioPlate = function()
	{
		var frm = document.forms['sample'];
		var url = '&resetTemporary=1';
		url += '&tmpfilter:INT:bioPlateType.bioMaterialType=|'+Data.get('page-data', 'sample-type');
		var subtypeId = ItemSubtype.getSubtypeId('subtype_id');
		// Restrict to plates with the given subtype
		url += '&tmpfilter:INT:bioPlateType.itemSubtype='+(subtypeId ? '|' + subtypeId : '=');
		url += '&tmpfilter:BOOLEAN:destroyed=false';
		Dialogs.selectItem('BIOPLATE', 'bioplate_id', 0, url);
	}
	
	samples.setBioPlateCallback = function(event)
	{
		var frm = document.forms['sample'];
		var list = frm.bioplate_id;
		if (list.length < 2 || list[1].value == '0') 
		{
			Forms.addListOption(list, 1, new Option());
		}
		list[1].value = event.detail.id;
		list[1].text = event.detail.name;
		list.selectedIndex = 1;
		samples.bioPlateOnChange();
	}

	samples.bioPlateOnChange = function()
	{
		var frm = document.forms['sample'];
		var list = frm.bioplate_id;
		frm.biowell_id.selectedIndex=0;
		frm.biowell_id.remove(1);			
	}
	
	samples.initBioWell = function()
	{
		var biowell = JSON.parse(Data.get('page-data', 'biowell'));
		if (biowell)
		{
			var frm = document.forms['sample'];
			var option = new Option(biowell.location, -biowell.id, true, true);
			frm.biowell_id[1] = option;
		}
	}
	
	samples.selectBioWell = function()
	{
		var frm = document.forms['sample'];
		var bioplateId = Math.abs(parseInt(frm.bioplate_id.value))
		if (bioplateId == 0)
		{
			Forms.showNotification('bioplate_id.select', 'You must first select a bioplate', null, 'pointer-below');
			return;
		}
		
		var url = '&bioplate_id='+bioplateId;
		url += '&resetTemporary=1&tmpfilter:STRING:$mbm.name='+escape('=');
		url += '&tmpfilter:STRING:originalBioMaterial.name='+escape('=');
		url += '&columns=row,column';

		Dialogs.selectItem('BIOWELL', 'biowell_id', 0, url);
	}
	samples.setBioWellCallback = function(event)
	{
		var frm = document.forms['sample'];
		var list = frm.biowell_id;
		if (list.length < 2 || list[1].value == '0') 
		{
			Forms.addListOption(list, 1, new Option());
		}
		list[1].value = event.detail.id;
		list[1].text = event.detail.name;
		list.selectedIndex = 1;
	}
	
	samples.loadAnnotationsFrame = function()
	{
		var frm = document.forms['sample'];
		var protocolId = 0;
		if (frm.protocol_id.length > 0 && !frm.protocol_id.disabled) 
		{
			protocolId = Math.abs(parseInt(frm.protocol_id.value));				
		}
		Annotations.autoLoadEditFrame(protocolId, ItemSubtype.getSubtypeId('subtype_id'));
	}

	samples.loadInheritedAnnotationsFrame = function()
	{
		Annotations.autoLoadInheritFrame(samples.getParents());
	}
	
	samples.getParents = function()
	{
		var frm = document.forms['sample'];
		var parents = [];
		if (frm.parentType[0].checked)
		{
			var bioSourceId = Math.abs(parseInt(frm.biosource_id.value));
			if (bioSourceId > 0) parents[parents.length] = 'BIOSOURCE:'+bioSourceId;
		}
		else
		{
			var ids = Link.getIdsInList(frm.samples, 'SAMPLE');
			if (ids.length > 0)
			{
				parents[parents.length] = 'SAMPLE:'+ids.join(':');
			}
		}
		return parents;
	}
	
	samples.selectProtocol = function()
	{
		var frm = document.forms['sample'];
		var url = '&resetTemporary=1';
		url += ItemSubtype.createRelatedFilter('subtype_id', 'PROTOCOL', Data.int('page-data', 'sampling-protocoltype-id'));
		if (frm.protocol_id.length > 1) 
		{
			var id = Math.abs(parseInt(frm.protocol_id[1].value));				
			url += '&item_id='+id;
		}
		Dialogs.selectItem('PROTOCOL', 'protocol_id', 0, url);
	}
	
	samples.setProtocolCallback = function(event)
	{
		var frm = document.forms['sample'];
		var list = frm.protocol_id;
		if (list.length < 2 || list[1].value == '0')
		{
			Forms.addListOption(list, 1, new Option());
		}
		list[1].value = event.detail.id;
		list[1].text = event.detail.name;
		list.selectedIndex = 1;
	}

	samples.subtypeOnChange = function()
	{
		var frm = document.forms['sample'];
		var subtypeId = ItemSubtype.getSubtypeId('subtype_id');
		var recentInfo = ItemSubtype.getRelatedProjectDefaultAndRecentItems('SAMPLE', subtypeId, ['PROTOCOL', 'BIOPLATE', 'BIOSOURCE', 'SAMPLE']);
		ItemSubtype.updateSelectionList(frm.protocol_id, recentInfo.PROTOCOL['recent'], recentInfo.PROTOCOL['default']);
		ItemSubtype.updateSelectionList(frm.bioplate_id, recentInfo.BIOPLATE['recent']);
	}

	samples.parentTypeOnClick = function()
	{
		var frm = document.forms['sample'];
		var useSamples = frm.parentType[1].checked;
		frm.biosource_id.disabled = useSamples;
		frm.samples.disabled = !useSamples;
		frm.used_quantity.disabled = !useSamples;
	}

	samples.selectBioSource = function()
	{
		var frm = document.forms['sample'];
		var url = '&resetTemporary=1';
		url += ItemSubtype.createRelatedFilter('subtype_id', 'BIOSOURCE');
		if (frm.biosource_id.length > 1) 
		{
			var id = Math.abs(parseInt(frm.biosource_id[1].value));				
			url += '&item_id='+id;
		}
		Dialogs.selectItem('BIOSOURCE', 'biosource_id', 0, url);
	}
	
	samples.setBioSourceCallback = function(event)
	{
		var frm = document.forms['sample'];
		var list = frm.biosource_id;
		if (list.length < 2 || list[1].value == '0')
		{
			Forms.addListOption(list, 1, new Option());
		}
		list[1].value = event.detail.id;
		list[1].text = event.detail.name;
		list.selectedIndex = 1;

		frm.parentType[0].checked = true;
		frm.parentType[1].checked = false;
		samples.parentTypeOnClick();
	}

	
	samples.addSamplesOnClick = function(event)
	{
		var frm = document.forms['sample'];
		var currentSamples = Link.getIdsInList(frm.samples, 'SAMPLE');
		var url = "&resetTemporary=1&exclude="+currentSamples.join(',');
		url += ItemSubtype.createRelatedFilter('subtype_id', 'SAMPLE');
		Dialogs.selectItem('SAMPLE', event.currentTarget.id, 1, url);
	}

	samples.addSampleCallback = function(event)
	{
		var frm = document.forms['sample'];
		event.detail.name += ' [-]';
		Link.addItem('samples', 'SAMPLE', event.detail);
		if (event.detail.remaining == 0)
		{
			frm.parentType[0].checked = false;
			frm.parentType[1].checked = true;
			samples.parentTypeOnClick();
		}
	}

	samples.usedQuantityOnChange = function()
	{
		var frm = document.forms['sample'];
		var usedQuantity = frm.used_quantity.value;
		var displayQuantity = usedQuantity == '' ? '-' : usedQuantity+' µg';
		for (var i = 0; i < frm.samples.length; i++)
		{
			var option = frm.samples[i];
			if (option.selected && option.item.id)
			{
				option.item.value = usedQuantity;
				var text = option.text.replace(/\[.*\]/, '['+displayQuantity+']');
				option.text = text; 
			}
		}
	}
	
	samples.samplesOnChange = function()
	{
		var frm = document.forms['sample'];
		var item = frm.samples[frm.samples.selectedIndex].item;
		if (item && item.id && item.value != undefined)
		{
			frm.used_quantity.value = item.value;
		}
		else
		{
			frm.used_quantity.value = '';
		}
		frm.used_quantity.focus();
	}
	
	return samples;
}();

Doc.addElementInitializer(Samples.initElements);
Doc.onLoad(Samples.initPage);
