/* $Id $
	------------------------------------------------------------------
	Copyright (C) 2012 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
*/

var ExpressionBuilder = function()
{

	var textarea;
	
	var builder = {};

	builder.initPage = function()
	{
		// Get the expression from the parent window
		textarea = window.opener.document.getElementById(Data.get('page-data', 'textarea'));
		
		var frm = document.forms['expression'];
		frm.expression.value = textarea.value;
		frm.expression.focus();
		
		Events.addEventHandler('spot', 'change', builder.insertSelected);
		Events.addEventHandler('rawdata', 'change', builder.insertSelected);
		Events.addEventHandler('reporter', 'change', builder.insertSelected);
		Events.addEventHandler('reporterList', 'change', builder.insertReporterListFunction);
		
		// Dialog buttons
		Buttons.addClickHandler('close', App.closeWindow);
		Buttons.addClickHandler('btnSave', builder.save);
		Buttons.addClickHandler('btnValidate', builder.validateExpression);
	}
	
	builder.initializeElements = function(element, autoInit)
	{
		if (autoInit == 'enclose-selection')
		{
			Buttons.addClickHandler(element, builder.textButtonOnClick);
		}
	}

	builder.save = function()
	{
		var value = document.forms['expression'].expression.value;
		if (textarea.type == 'text') value = value.replace(/\n/g, '');
		textarea.value = value;
		Events.sendChangeEvent(textarea);
		window.close();
	}
	
	builder.textButtonOnClick = function(event)
	{		
		var prefix = Data.get(event.currentTarget, 'prefix') || '';
		var suffix = Data.get(event.currentTarget, 'suffix') || '';
		builder.encloseSelection(prefix, suffix);
	}

	builder.insertSelected = function(event)
	{
		var list = event.currentTarget;
		var selected = list[list.selectedIndex];
		
		var prefix = Data.get(selected, 'prefix') || '';
		var suffix = Data.get(selected, 'suffix') || '';
		
		builder.encloseSelection(prefix, suffix);
		list.selectedIndex = 0;
	}
	
	builder.insertReporterListFunction = function()
	{
		var frm = document.forms['expression'];
		var listFunction = frm.listFunction.value;
		var listId = frm.reporterList.value;
		builder.encloseSelection(listFunction+'(' + listId + ')', '');
		frm.reporterList.selectedIndex = 0;
	}
	
	builder.encloseSelection = function(prefix, suffix)
	{
		
		/*
			Enclose the selected text with prefix and suffix. Code taken
			from Trac: http://projects.edgewall.com/trac/
		*/
		var frm = document.forms['expression'];
		var textarea = frm.expression;
		textarea.focus();
	    var start, end, sel, scrollPos, subst;
	    if (typeof(document["selection"]) != "undefined") 
	    {
	      sel = document.selection.createRange().text;
	    } 
	    else if (typeof(textarea["setSelectionRange"]) != "undefined") 
	    {
	      start = textarea.selectionStart;
	      end = textarea.selectionEnd;
	      scrollPos = textarea.scrollTop;
	      sel = textarea.value.substring(start, end);
	    }
	    if (sel.match(/ $/)) 
	    { 
	    	// exclude ending space char, if any
	      	sel = sel.substring(0, sel.length - 1);
	      	suffix = suffix + " ";
	    }
	    subst = prefix + sel + suffix;
	    if (typeof(document["selection"]) != "undefined") 
	    {
	      var range = document.selection.createRange().text = subst;
	      textarea.caretPos -= suffix.length;
	    } 
	    else if (typeof(textarea["setSelectionRange"]) != "undefined") 
	    {
	      textarea.value = textarea.value.substring(0, start) + subst +
	                       textarea.value.substring(end);
	      if (sel) 
	      {
	        textarea.setSelectionRange(start + subst.length, start + subst.length);
	      } 
	      else 
	      {
	        textarea.setSelectionRange(start + prefix.length, start + prefix.length);
	      }
	      textarea.scrollTop = scrollPos;
	    }
	}

	builder.validateExpression = function()
	{
		var frm = document.forms['expression'];
		var expression = frm.expression.value;
		
		var request = Ajax.getXmlHttpRequest();
		var url = 'expression_builder_ajax.jsp?ID='+App.getSessionId();
		url += '&cmd=ValidateExpression&expression=' + encodeURIComponent(expression);
		url += '&formulatype='+Data.get('page-data', 'formula-type');
		url += '&rawdatatype='+Data.get('page-data', 'raw-data-type');
		url += '&channels='+Data.get('page-data', 'channels');
		
		request.open("GET", url, false);
		request.send(null);

		var response = JSON.parse(request.responseText);
		var message;
		if (response.status != 'ok')
		{
			message = response.message;
			message = message.replace('org.nfunk.jep.ParseException:', '');
		}
		else 
		{
			message = 'The expression seems to be ok.';
		}
		Forms.showNotification('btnValidate', message + '<br><span style="font-weight: normal;">Note! This feature is experimental and may not be 100% correct.</span>');
		
	}
	
	return builder;
}();

Doc.onLoad(ExpressionBuilder.initPage);
Doc.addElementInitializer(ExpressionBuilder.initializeElements);
